from typing import Optional
from enum import Enum
from dataclasses import dataclass


class EnvType(Enum):

    DOTENV = 1
    DOCKERFILE = 2
    DOCKER_COMPOSE = 3
    NONE = 4


@dataclass(frozen=False, order=True)
class EnvPaths:
    docker_compose_path: Optional[str]
    dotenv_path: Optional[str]
    dockerfile_path: Optional[str]
    python_env_path: Optional[str]



import re as __re


def __indent(__str: str):
    return len(__str) - len(__str.lstrip())


def __first_letter_match(__str: str):
    return __re.search("[a-zA-Z]", __str)


def __from_match_start(__str: str, __match: __re.Match):
    return __str[__match.start() if __match else 0 :]


def __from_first_letter(__str: str):
    match = __first_letter_match(__str)
    return __from_match_start(__str, match)


def from_first_letter(__str: str):
    return __from_first_letter(__str)


def get_indent(__str: str):
    return __indent(__str)



from ..utils import string_utils


def docker_compose_lines(__docker_compose_src: str):
    indent, results = None, []
    for line in __docker_compose_src.splitlines(keepends=False):
        line_indent = string_utils.get_indent(line)
        if indent is None:
            if line.strip().endswith("environment:"):
                indent = line_indent
        elif line_indent > indent:
            ffl = string_utils.from_first_letter(line)
            if "=" in ffl:
                results.append(ffl)
            elif ":" in ffl:
                lhs, rhs = ffl.split(":", 1)
                ffl = f"{lhs.rstrip()}={rhs.lstrip()}"
                results.append(ffl)
        else:
            indent = None
    return results


def dotenv_lines(__src: str):
    return [line for line in __src.splitlines(keepends=False) if "=" in line]


def dockerfile_lines(__src: str):
    return [
        string_utils.from_first_letter(line.removeprefix("ENV"))
        for line in __src.splitlines(keepends=False)
        if line.lstrip().startswith("ENV") and "=" in line
    ]



def __safe_merge(__lhs: dict, __rhs: dict):
    result = __lhs.copy()
    for key, value in __rhs.items():
        if key in __lhs:
            lvalue = __lhs.get(key)
            if lvalue != value:
                raise Exception(
                    f"ERROR: CONFLICT OF DICT KEYS:: {str(key)} : {lvalue} : {value}"
                )
        result[key] = value
    return result


def safe_merge(__lhs: dict, __rhs: dict):
    return __safe_merge(__lhs, __rhs)



from typing import Optional
from pathlib import Path


def path_exists(__path: Optional[str]):
    return __path is not None and Path(__path).exists()


def read_text(__path: Optional[str]):
    return Path(__path).read_text()


def write_text(path: Optional[str], data: str):
    Path(path).write_text(data=data)



from typing import Optional
from .objects import EnvPaths, EnvType
from .core import env_dict
from .core import env_src
from .utils import dict_utils
from .utils import path_utils


def docker_compose_dict(path: str):
    return env_dict.env_dict(path=path, env=EnvType.DOCKER_COMPOSE)


def dotenv_dict(path: str):
    return env_dict.env_dict(path=path, env=EnvType.DOTENV)


def dockerfile_dict(path: str):
    return env_dict.env_dict(path=path, env=EnvType.DOCKERFILE)


def unified_environment(paths: EnvPaths):
    result = {}
    path = paths.docker_compose_path
    if path_utils.path_exists(path):
        docker_compose_env = docker_compose_dict(path=path)
        result = dict_utils.safe_merge(result, docker_compose_env)
    path = paths.dockerfile_path
    if path_utils.path_exists(path):
        dockerfile_env = dockerfile_dict(path)
        result = dict_utils.safe_merge(result, dockerfile_env)
    path = paths.dotenv_path
    if path_utils.path_exists(path):
        dotenv_env = dotenv_dict(path)
        result = dict_utils.safe_merge(result, dotenv_env)
    return result


def update_envs(
    docker_compose_path: Optional[str] = None,
    dotenv_path: Optional[str] = None,
    dockerfile_path: Optional[str] = None,
    python_env_path: Optional[str] = "env.py",
):

    env_paths = EnvPaths(
        **dict(
            docker_compose_path=docker_compose_path,
            dotenv_path=dotenv_path,
            dockerfile_path=dockerfile_path,
            python_env_path=python_env_path,
        )
    )

    env = unified_environment(paths=env_paths)

    if env:

        if path_utils.path_exists(dotenv_path):
            dotenv_src = env_src.form_dotenv_src(env)
            path_utils.write_text(dotenv_path, dotenv_src)
        if path_utils.path_exists(python_env_path):
            if path_utils.path_exists(dotenv_path):
                python_src = env_src.form_python_env_src(
                    env=env, dotenv_path=dotenv_path, python_env_path=python_env_path
                )
                path_utils.write_text(python_env_path, python_src)
        if path_utils.path_exists(dockerfile_path):
            dockerfile_src = env_src.form_dockerfile_env_src(env, dockerfile_path)
            path_utils.write_text(dockerfile_path, dockerfile_src)
